module Scheduler where

import Prelude hiding (log)
import qualified Data.Map as M
import Data.Map (Map)

type Time = Double
type DiffTime = Double

type Callback = DiffTime {- How early the callback is, in fractions of a sample -} -> Scheduler -> Scheduler

type Engine = Integer {- Number of samples to run for -} -> Scheduler -> Scheduler

type Message = String

data Scheduler = Scheduler { now :: Time, future :: Map Time Callback, engine :: Engine, log :: [Message] }

startTime = 0
tickDelta = 123.456
tickTime = startTime + tickDelta
runTick dt s = s { future = M.insert (now s + tickTime + dt) runTick (future s) }

startTime2 = 0
tickDelta2 = 76.5432
tickTime2 = startTime2 + tickDelta2
runTick2 dt s = s { future = M.insert (now s + tickTime2 + dt) runTick2 (future s) }


runEngine n = id
scheduler0 = Scheduler { now = 0, future = M.insert tickTime runTick (M.insert tickTime2 runTick2 M.empty), engine = runEngine, log = [] }

runScheduler :: Scheduler -> Scheduler
runScheduler s = let Just ((stepTime, stepEvent), f) = M.minViewWithKey (future s)
                     samples = floor $ stepTime - now s
                     s' = (engine s) samples (s{ now = now s + fromIntegral samples, future = f, log = log s ++ [ "Engine: " ++ show samples ] })
                     delta = stepTime - now s'
                     s'' = stepEvent delta (s'{ log = log s' ++ [ "Event: " ++ show delta ] })
                 in s''

test = log ((iterate runScheduler scheduler0) !! 10)

